@@ -421,12 +421,13 @@ def __post_for_token(iam_server: IamServer,
                                                             data=body_data)
                 if response.status_code == 200:
                     # request succeeded
-                    if logger:
-                        logger.debug(msg=f"POST success, status {response.status_code}")
                     result = response.json()
+                    if logger:
+                        logger.debug(msg=f"POST success, {json.dumps(obj=result,
+                                                                     ensure_ascii=False)}")
                 else:
                     # request resulted in error
-                    err_msg = f"POST failure, status {response.status_code}, reason '{response.reason}'"
+                    err_msg = f"POST failure, status {response.status_code}, reason {response.reason}"
                     if hasattr(response, "content") and response.content:
                         err_msg += f", content '{response.content}'"
                     if logger:
@@ -502,7 +503,8 @@ def __validate_and_store(iam_server: IamServer,
                 users: dict[str, dict[str, Any]] = _get_iam_users(iam_server=iam_server,
                                                                   errors=errors,
                                                                   logger=logger)
-                if users:
+                # must test with 'not errors'
+                if not errors:
                     user_id: str = login_id if login_id else claims["payload"][recipient_attr]
                     users[user_id] = user_data
                     result = (user_id, token)
