@@ -1,10 +1,11 @@
+import json
 import requests
 import sys
 from base64 import b64encode
 from datetime import datetime
 from logging import Logger
 from pypomes_core import TZ_LOCAL, exc_format
-from threading import Lock
+from threading import RLock
 from typing import Any, Final
 
 # structure:
@@ -16,15 +17,15 @@ from typing import Any, Final
 #      "basic-auth": <bool>,
 #      "headers-data": <dict[str, str]>,
 #      "body-data": <dict[str, str],
-#      "token": <str>,
-#      "expiration": <timestamp>
+#      "access-token": <str>,
+#      "access-expiration": <timestamp>
 #    }
 # }
 _provider_registry: Final[dict[str, dict[str, Any]]] = {}
 
 # the lock protecting the data in '_provider_registry'
 # (because it is 'Final' and set at declaration time, it can be accessed through simple imports)
-_provider_lock: Final[Lock] = Lock()
+_provider_lock: Final[RLock] = RLock()
 
 
 def provider_register(provider_id: str,
@@ -63,8 +64,10 @@ def provider_register(provider_id: str,
             "custom-auth": custom_auth,
             "headers-data": headers_data,
             "body-data": body_data,
-            "token": None,
-            "expiration": datetime.now(tz=TZ_LOCAL).timestamp()
+            "access-token": None,
+            "access-expiration": 0,
+            "refresh-token": None,
+            "refresh-expiration": 0
         }
 
 
@@ -88,7 +91,7 @@ def provider_get_token(provider_id: str,
         provider: dict[str, Any] = _provider_registry.get(provider_id)
         if provider:
             now: float = datetime.now(tz=TZ_LOCAL).timestamp()
-            if now > provider.get("expiration"):
+            if now > provider.get("access-expiration"):
                 user: str = provider.get("user")
                 pwd: str = provider.get("pwd")
                 headers_data: dict[str, str] = provider.get("headers-data") or {}
@@ -101,6 +104,9 @@ def provider_get_token(provider_id: str,
                     enc_bytes: bytes = b64encode(f"{user}:{pwd}".encode())
                     headers_data["Authorization"] = f"Basic {enc_bytes.decode()}"
                 url: str = provider.get("url")
+            if logger:
+                logger.debug(msg=f"POST {url}, {json.dumps(obj=body_data,
+                                                           ensure_ascii=False)}")
                 try:
                     # typical return on a token request:
                     # {
@@ -117,19 +123,28 @@ def provider_get_token(provider_id: str,
                                                                 timeout=None)
                     if response.status_code < 200 or response.status_code >= 300:
                         # request resulted in error, report the problem
-                        err_msg = (f"POST '{url}': failed, "
-                                   f"status {response.status_code}, reason '{response.reason}'")
+                        err_msg = (f"POST failure, "
+                                   f"status {response.status_code}, reason {response.reason}")
                     else:
-                        reply: dict[str, Any] = response.json()
-                        provider["token"] = reply.get("access_token")
-                        provider["expiration"] = now + int(reply.get("expires_in"))
+                        # request succeeded
                         if logger:
-                            logger.debug(msg=f"POST '{url}': status {response.status_code}")
+                            logger.debug(msg=f"POST success, status {response.status_code}")
+                        reply: dict[str, Any] = response.json()
+                        provider["access-token"] = reply.get("access_token")
+                        provider["access-expiration"] = now + int(reply.get("expires_in"))
+                        if reply.get("refresh_token"):
+                            provider["refresh-token"] = reply["refesh_token"]
+                            if reply.get("refresh_expires_in"):
+                                provider["refresh-expiration"] = now + int(reply.get("refresh_expires_in"))
+                            else:
+                                provider["refresh-expiration"] = sys.maxsize
+                        if logger:
+                            logger.debug(msg=f"POST {url}: status {response.status_code}")
                 except Exception as e:
                     # the operation raised an exception
                     err_msg = exc_format(exc=e,
                                          exc_info=sys.exc_info())
-                    err_msg = f"POST '{url}': error, '{err_msg}'"
+                    err_msg = f"POST error, '{err_msg}'"
         else:
             err_msg: str = f"Provider '{provider_id}' not registered"
 
@@ -139,7 +154,7 @@ def provider_get_token(provider_id: str,
         if logger:
             logger.error(msg=err_msg)
     else:
-        result = provider.get("token")
+        result = provider.get("access-token")
 
     return result
 
