@@ -9,6 +9,8 @@ from typing import Any
 
 def token_validate(token: str,
                    issuer: str = None,
+                   recipient_id: str = None,
+                   recipient_attr: str = None,
                    public_key: str | bytes | PyJWK | RSAPublicKey = None,
                    errors: list[str] = None,
                    logger: Logger = None) -> dict[str, dict[str, Any]] | None:
@@ -24,15 +26,21 @@ def token_validate(token: str,
     If an asymmetric algorithm was used to sign the token and *public_key* is provided, then
     the token is validated, by using the data in its *signature* section.
 
+    The parameters *recipient_id* and *recipient_attr* refer the token's expected subject, respectively,
+    the subject's identification and the attribute in the token's payload data identifying its subject.
+    If both are provided, *recipient_id* is validated.
+
     On failure, *errors* will contain the reason(s) for rejecting *token*.
     On success, return the token's claims (*header* and *payload*).
 
     :param token: the token to be validated
     :param public_key: optional public key used to sign the token, in *PEM* format
     :param issuer: optional value to compare with the token's *iss* (issuer) attribute in its *payload*
+    :param recipient_id: identification of the expected token subject
+    :param recipient_attr: attribute in the token's payload holding the expected subject's identification
     :param errors: incidental error messages
     :param logger: optional logger
-    :return: The token's claims (*header* and *payload*) if it is valid, *None* otherwise
+    :return: The token's claims (*header* and *payload*), or *None* if error
     """
     # initialize the return variable
     result: dict[str, dict[str, Any]] | None = None
@@ -87,10 +95,17 @@ def token_validate(token: str,
                                                  algorithms=[token_alg],
                                                  options=options,
                                                  issuer=issuer)
-            result = {
-                "header": token_header,
-                "payload": payload
-            }
+            if recipient_id and recipient_attr and \
+                    payload.get(recipient_attr) and recipient_id != payload.get(recipient_attr):
+                msg: str = f"Token was issued to '{payload.get(recipient_attr)}', not to '{recipient_id}'"
+                if logger:
+                    logger.error(msg=msg)
+                errors.append(msg)
+            else:
+                result = {
+                    "header": token_header,
+                    "payload": payload
+                }
         except Exception as e:
             exc_err: str = exc_format(exc=e,
                                       exc_info=sys.exc_info())
